@@ -10,9 +10,10 @@ import os
 import subprocess
 import re
 import logging
+import unicodedata
+from functools import partial
 from tryton.config import CONFIG
 from tryton.config import TRYTON_ICON, PIXMAPS_DIR
-import time
 import sys
 import xmlrpclib
 try:
@@ -37,10 +38,7 @@ try:
 except ImportError:
     ssl = None
 from threading import Lock
-try:
-    import pytz
-except ImportError:
-    pytz = None
+import dateutil.tz
 
 from tryton.exceptions import (TrytonServerError, TrytonError,
     TrytonServerUnavailable)
@@ -170,8 +168,25 @@ class ModelAccess(object):
 MODELACCESS = ModelAccess()
 
 
+class ModelHistory(object):
+    _models = set()
+
+    def load_history(self):
+        self._models.clear()
+        try:
+            self._models.update(rpc.execute('model', 'ir.model',
+                    'list_history', rpc.CONTEXT))
+        except TrytonServerError:
+            pass
+
+    def __contains__(self, model):
+        return model in self._models
+
+MODELHISTORY = ModelHistory()
+
+
 class ViewSearch(object):
-    searches = None
+    searches = {}
 
     def load_searches(self):
         try:
@@ -248,8 +263,6 @@ def refresh_langlist(lang_widget, host, port):
         lang_list = rpc.db_exec(host, port, 'list_lang')
     except socket.error:
         return []
-    from tryton.gui.main import Main
-    Main.get_main().refresh_ssl()
     index = -1
     i = 0
     lang = locale.getdefaultlocale()[0]
@@ -436,7 +449,11 @@ def file_selection(title, filename='',
     win.set_icon(TRYTON_ICON)
     win.set_current_folder(CONFIG['client.default_path'])
     if filename:
-        win.set_current_name(filename)
+        if action in (gtk.FILE_CHOOSER_ACTION_SAVE,
+                gtk.FILE_CHOOSER_ACTION_CREATE_FOLDER):
+            win.set_current_name(filename)
+        else:
+            win.set_filename(filename)
     win.set_select_multiple(multi)
     win.set_default_response(gtk.RESPONSE_OK)
     if filters is not None:
@@ -448,8 +465,8 @@ def file_selection(title, filename='',
         filename = win.get_preview_filename()
         if filename:
             try:
-                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(filename.decode(
-                        sys.getfilesystemencoding().encode('utf-8')), 128, 128)
+                pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(
+                    filename, 128, 128)
                 img.set_from_pixbuf(pixbuf)
                 have_preview = True
             except (IOError, glib.GError):
@@ -494,6 +511,18 @@ def file_selection(title, filename='',
         return filenames
 
 
+_slugify_strip_re = re.compile(r'[^\w\s-]')
+_slugify_hyphenate_re = re.compile(r'[-\s]+')
+
+
+def slugify(value):
+    if not isinstance(value, unicode):
+        value = unicode(value)
+    value = unicodedata.normalize('NFKD', value).encode('ascii', 'ignore')
+    value = unicode(_slugify_strip_re.sub('', value).strip().lower())
+    return _slugify_hyphenate_re.sub('-', value)
+
+
 def file_open(filename, type, print_p=False):
     def save():
         save_name = file_selection(_('Save As...'),
@@ -624,12 +653,8 @@ class WarningDialog(UniqueDialog):
     def build_dialog(self, parent, message, title, buttons=gtk.BUTTONS_OK):
         dialog = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT,
             gtk.MESSAGE_WARNING, buttons)
-        if hasattr(dialog, 'format_secondary_markup'):
-            dialog.set_markup('<b>%s</b>' % (to_xml(title)))
-            dialog.format_secondary_markup(to_xml(message))
-        else:
-            dialog.set_markup(
-                '<b>%s</b>\n%s' % (to_xml(title), to_xml(message)))
+        dialog.set_markup('<b>%s</b>' % (to_xml(title)))
+        dialog.format_secondary_markup(to_xml(message))
         return dialog
 
 warning = WarningDialog()
@@ -971,9 +996,10 @@ def send_bugtracker(title, msg):
             protocol = 'http'
             if ssl or hasattr(socket, 'ssl'):
                 protocol = 'https'
+            quote = partial(urllib.quote, safe="!$&'()*+,;=:")
             server = xmlrpclib.Server(
                 ('%s://%s:%s@' + CONFIG['roundup.xmlrpc'])
-                % (protocol, user, password), allow_none=True)
+                % (protocol, quote(user), quote(password)), allow_none=True)
             if hashlib:
                 msg_md5 = hashlib.md5(msg + '\n' + title).hexdigest()
             else:
@@ -982,10 +1008,15 @@ def send_bugtracker(title, msg):
                 title = '[no title]'
             issue_id = None
             msg_ids = server.filter('msg', None, {'summary': str(msg_md5)})
-            if msg_ids:
-                issue_ids = server.filter('issue', None, {'messages': msg_ids})
-                if issue_ids:
-                    issue_id = issue_ids[0]
+            for msg_id in msg_ids:
+                summary = server.display(
+                    'msg%s' % msg_id, 'summary')['summary']
+                if summary == msg_md5:
+                    issue_ids = server.filter(
+                        'issue', None, {'messages': msg_id})
+                    if issue_ids:
+                        issue_id = issue_ids[0]
+                        break
             if issue_id:
                 # issue to same message already exists, add user to nosy-list
                 server.set('issue' + str(issue_id), *['nosy=+' + user])
@@ -1047,7 +1078,7 @@ def process_exception(exception, *args, **kwargs):
             name, msg, description = exception.args
             res = userwarning(description, msg)
             if res in ('always', 'ok'):
-                args2, = ('model', 'res.user.warning', 'create', [{
+                args2 = ('model', 'res.user.warning', 'create', [{
                             'user': rpc._USER,
                             'name': name,
                             'always': (res == 'always'),
@@ -1104,7 +1135,6 @@ def process_exception(exception, *args, **kwargs):
                         return False
                     res = rpc.login(rpc._USERNAME, password, hostname, port,
                             rpc._DATABASE)
-                    Main.get_main().refresh_ssl()
                     if res == -1:
                         message(_('Connection error!\n'
                                 'Unable to connect to the server!'))
@@ -1194,20 +1224,18 @@ def generateColorscheme(masterColor, keys, light=0.06):
 class DBProgress(object):
 
     def __init__(self, host, port):
-        self.dbs, self.createdb = None, None
+        self.dbs = None, None
         self.host, self.port = host, port
         self.updated = threading.Event()
-        self.db_info = None
 
     def start(self):
-        dbs, createdb = None, False
+        dbs = None
         try:
             dbs = refresh_dblist(self.host, self.port)
-            createdb = True
         except Exception:
             pass
         finally:
-            self.db_info = (dbs, createdb)
+            self.dbs = dbs
             self.updated.set()
 
     def update(self, combo, progressbar, callback, dbname=''):
@@ -1222,15 +1250,9 @@ class DBProgress(object):
             progressbar.pulse()
             return True
         progressbar.hide()
-        dbs, createdb = self.db_info
+        dbs = self.dbs
 
-        if dbs is None:
-            dbs, createdb = None, False
-        elif dbs == -1:
-            dbs, createdb = -1, False
-        else:
-            from tryton.gui.main import Main
-            Main.get_main().refresh_ssl()
+        if dbs is not None and dbs not in (-1, -2):
             liststore = combo.get_model()
             liststore.clear()
             index = -1
@@ -1243,7 +1265,7 @@ class DBProgress(object):
             combo.set_active(index)
             dbs = len(dbs)
 
-        callback(dbs, createdb)
+        callback(dbs)
         return False
 
 
@@ -1270,42 +1292,55 @@ class RPCProgress(object):
             self.error = True
             self.res = False
             self.exception = exception
-            return True
-        if not self.res:
-            self.error = True
+        else:
+            if not self.res:
+                self.error = True
+        if self.callback:
+            # Post to GTK queue to be run by the main thread
+            gobject.idle_add(self.process)
         return True
 
-    def run(self, process_exception_p=True, main_iteration_p=True):
-        session = rpc._SESSION
-        thread.start_new_thread(self.start, ())
+    def run(self, process_exception_p=True, callback=None):
+        self.process_exception_p = process_exception_p
+        self.callback = callback
+        self.session = rpc._SESSION
+        if self.parent.window:
+            watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
+            self.parent.window.set_cursor(watch)
 
-        watch = None
-        i = 0
-        while (not self.res) and (not self.error):
-            i += 1
-            if i > 1:
-                watch = gtk.gdk.Cursor(gtk.gdk.WATCH)
-                self.parent.window.set_cursor(watch)
-                if main_iteration_p:
-                    with gtk.gdk.lock:
-                        while gtk.events_pending():
-                            gtk.main_iteration()
-            time.sleep(0.1)
+        if callback:
+            thread.start_new_thread(self.start, ())
+            return
+        else:
+            self.start()
+            return self.process()
+
+    def process(self):
         if self.parent.window:
             self.parent.window.set_cursor(None)
         if self.exception:
-            if process_exception_p:
+            if self.process_exception_p:
                 def rpc_execute(*args):
                     return RPCProgress('execute',
-                        args).run(process_exception_p, main_iteration_p)
+                        args).run(self.process_exception_p)
                 result = process_exception(self.exception, *self.args,
-                    rpc_execute=rpc_execute, session=session)
+                    rpc_execute=rpc_execute, session=self.session)
                 if result is False:
-                    raise RPCException(self.exception)
-                return result
-            else:
+                    self.exception = RPCException(self.exception)
+                else:
+                    self.exception = None
+                    self.res = result
+
+        def return_():
+            if self.exception:
                 raise self.exception
-        return self.res
+            else:
+                return self.res
+
+        if self.callback:
+            self.callback(return_)
+        else:
+            return return_()
 
 
 def RPCExecute(*args, **kwargs):
@@ -1314,8 +1349,19 @@ def RPCExecute(*args, **kwargs):
         rpc_context.update(kwargs['context'])
     args = args + (rpc_context,)
     process_exception = kwargs.get('process_exception', True)
-    main_iteration = kwargs.get('main_iteration', True)
-    return RPCProgress('execute', args).run(process_exception, main_iteration)
+    callback = kwargs.get('callback')
+    return RPCProgress('execute', args).run(process_exception, callback)
+
+
+def RPCContextReload(callback=None):
+    def update(context):
+        rpc.CONTEXT.clear()
+        rpc.CONTEXT.update(context())
+        if callback:
+            callback()
+    # Use RPCProgress to not send rpc.CONTEXT
+    RPCProgress('execute', ('model', 'res.user', 'get_preferences', True, {})
+        ).run(True, update)
 
 
 class Tooltips(object):
@@ -1472,15 +1518,11 @@ def filter_domain(domain):
 
 
 def timezoned_date(date, reverse=False):
-    if pytz and rpc.CONTEXT.get('timezone') and rpc.TIMEZONE:
-        lzone = pytz.timezone(rpc.CONTEXT['timezone'])
-        szone = pytz.timezone(rpc.TIMEZONE)
-        if reverse:
-            lzone, szone = szone, lzone
-        sdt = szone.localize(date, is_dst=True)
-        ldt = sdt.astimezone(lzone)
-        date = ldt
-    return date
+    lzone = dateutil.tz.tzlocal()
+    szone = dateutil.tz.tzutc()
+    if reverse:
+        lzone, szone = szone, lzone
+    return date.replace(tzinfo=szone).astimezone(lzone)
 
 
 def untimezoned_date(date):
