@@ -123,6 +123,8 @@ class DBCreate(object):
             entry.handler_unblock(_hid)
 
             def _move_cursor():
+                if not entry.props.window:
+                    return
                 with gtk.gdk.lock:
                     entry.set_position(_pos)
                     return False
@@ -396,11 +398,7 @@ class DBCreate(object):
                             parent.present()
                             self.dialog.destroy()
                             rpc.logout()
-                            from tryton.gui.main import Main
-                            Main.get_main().refresh_ssl()
                             break
-                        from tryton.gui.main import Main
-                        Main.get_main().refresh_ssl()
                         parent.present()
                         self.dialog.destroy()
                         if self.sig_login:
