@@ -3,16 +3,20 @@
 "Screen"
 import copy
 import gobject
+import datetime
+import calendar
 try:
     import simplejson as json
 except ImportError:
     import json
 import collections
+import urllib
+import urlparse
 import xml.dom.minidom
 import gettext
+import logging
 
 from tryton.gui.window.view_form.model.group import Group
-from tryton.gui.window.view_form.model.record import Record
 from tryton.gui.window.view_form.view.screen_container import ScreenContainer
 from tryton.signal_event import SignalEvent
 from tryton.config import CONFIG
@@ -20,8 +24,9 @@ from tryton.exceptions import TrytonServerError, TrytonServerUnavailable
 from tryton.jsonrpc import JSONEncoder
 from tryton.common.domain_parser import DomainParser
 from tryton.common import RPCExecute, RPCException, MODELACCESS, \
-    node_attributes, sur
+    node_attributes, sur, RPCContextReload
 from tryton.action import Action
+import tryton.rpc as rpc
 
 _ = gettext.gettext
 
@@ -35,20 +40,18 @@ class Screen(SignalEvent):
 
     def __init__(self, model_name, view_ids=None, mode=None, context=None,
             views_preload=None, domain=None, row_activate=None, limit=None,
-            readonly=False, exclude_field=None, sort=None, search_value=None,
+            readonly=False, exclude_field=None, order=None, search_value=None,
             tab_domain=None, alternate_view=False):
         if view_ids is None:
             view_ids = []
         if mode is None:
             mode = ['tree', 'form']
-        if context is None:
-            context = {}
         if views_preload is None:
             views_preload = {}
         if domain is None:
             domain = []
 
-        self.limit = limit or int(CONFIG['client.limit'])
+        self.limit = limit or CONFIG['client.limit']
         self.offset = 0
         super(Screen, self).__init__()
 
@@ -64,15 +67,17 @@ class Screen(SignalEvent):
         self.size_limit = None
         self.views_preload = views_preload
         self.model_name = model_name
-        self.context = context
         self.views = []
         self.view_ids = view_ids[:]
         self.parent = None
         self.parent_name = None
         self.exclude_field = exclude_field
         self.filter_widget = None
+        self.tree_states = collections.defaultdict(
+            lambda: collections.defaultdict(lambda: None))
+        self.tree_states_done = set()
         self.__group = None
-        self.new_group()
+        self.new_group(context or {})
         self.__current_record = None
         self.current_record = None
         self.screen_container = ScreenContainer(tab_domain)
@@ -81,10 +86,8 @@ class Screen(SignalEvent):
         self.__current_view = 0
         self.search_value = search_value
         self.fields_view_tree = None
-        self.sort = sort
+        self.order = order
         self.view_to_load = []
-        self.expanded_nodes = collections.defaultdict(
-            lambda: collections.defaultdict(lambda: None))
         self.domain_parser = None
         self.pre_validate = False
         self.view_to_load = mode[:]
@@ -120,17 +123,23 @@ class Screen(SignalEvent):
                 xml_fields = [node_attributes(node).get('name')
                     for node in root_node.childNodes
                     if node.nodeName == 'field']
-                if hasattr(collections, 'OrderedDict'):
-                    odict = collections.OrderedDict
-                else:
-                    odict = dict
-                fields = odict((name, fields[name]) for name in xml_fields)
-                if 'id' not in fields:
-                    fields['id'] = {
-                        'string': _('ID'),
-                        'name': 'id',
-                        'type': 'integer',
-                        }
+                fields = collections.OrderedDict(
+                    (name, fields[name]) for name in xml_fields)
+                for name, string, type_ in (
+                        ('id', _('ID'), 'integer'),
+                        ('create_uid', _('Creation User'), 'many2one'),
+                        ('create_date', _('Creation Date'), 'datetime'),
+                        ('write_uid', _('Modification User'), 'many2one'),
+                        ('write_date', _('Modification Date'), 'datetime'),
+                        ):
+                    if name not in fields:
+                        fields[name] = {
+                            'string': string.decode('utf-8'),
+                            'name': name,
+                            'type': type_,
+                            }
+                        if type_ == 'datetime':
+                            fields[name]['format'] = '"%H:%M:%S"'
 
                 self.domain_parser = DomainParser(fields)
 
@@ -182,13 +191,19 @@ class Screen(SignalEvent):
         else:
             domain = self.domain
 
+        if self.current_view.view_type == 'calendar':
+            if domain:
+                domain = ['AND', domain, self.current_view.current_domain()]
+            else:
+                domain = self.current_view.current_domain()
+
         tab_domain = self.screen_container.get_tab_domain()
         if tab_domain:
             domain = ['AND', domain, tab_domain]
 
         try:
             ids = RPCExecute('model', self.model_name, 'search', domain,
-                self.offset, self.limit, self.sort, context=self.context)
+                self.offset, self.limit, self.order, context=self.context)
         except RPCException:
             ids = []
         if not only_ids:
@@ -212,6 +227,10 @@ class Screen(SignalEvent):
         self.load(ids)
         return bool(ids)
 
+    @property
+    def context(self):
+        return self.group.context
+
     def __get_group(self):
         return self.__group
 
@@ -221,6 +240,7 @@ class Screen(SignalEvent):
             self.group.signal_unconnect(self)
             for name, field in self.group.fields.iteritems():
                 fields[name] = field.attrs
+        self.tree_states_done.clear()
         self.__group = group
         self.parent = group.parent
         self.parent_name = group.parent_name
@@ -241,9 +261,10 @@ class Screen(SignalEvent):
 
     group = property(__get_group, __set_group)
 
-    def new_group(self):
+    def new_group(self, context=None):
+        context = context if context is not None else self.context
         self.group = Group(self.model_name, {}, domain=self.domain,
-            context=self.context, readonly=self.readonly)
+            context=context, readonly=self.readonly)
 
     def _group_cleared(self, group, signal):
         for view in self.views:
@@ -301,16 +322,10 @@ class Screen(SignalEvent):
         return False
 
     def destroy(self):
-        self.save_tree_state()
         self.group.destroy()
         for view in self.views:
             view.destroy()
         super(Screen, self).destroy()
-        self.parent = None
-        self.__group = None
-        self.__current_record = None
-        self.screen_container = None
-        self.widget = None
 
     def default_row_activate(self):
         if (self.current_view.view_type == 'tree' and
@@ -337,7 +352,8 @@ class Screen(SignalEvent):
                     self.current_record not in self.current_record.group):
                 self.current_record = None
             fields = self.current_view.get_fields()
-            if self.current_record and not self.current_record.validate(fields):
+            if (self.current_record and self.current_view.editable
+                    and not self.current_record.validate(fields)):
                 self.screen_container.set(self.current_view.widget)
                 self.set_cursor()
                 self.current_view.display()
@@ -355,7 +371,6 @@ class Screen(SignalEvent):
                 elif self.current_view.view_type == view_type:
                     break
         self.screen_container.set(self.current_view.widget)
-        self.current_view.cancel()
         self.display()
         self.set_cursor()
 
@@ -384,6 +399,7 @@ class Screen(SignalEvent):
     def add_view(self, view):
         arch = view['arch']
         fields = view['fields']
+        view_id = view['view_id']
 
         xml_dom = xml.dom.minidom.parseString(arch)
         for node in xml_dom.childNodes:
@@ -412,23 +428,16 @@ class Screen(SignalEvent):
         parser = WidgetParse(parent=self.parent)
         view = parser.parse(self, xml_dom, self.group.fields,
                 children_field=children_field)
+        view.view_id = view_id
 
         self.views.append(view)
         return view
 
-    def editable_get(self):
-        if hasattr(self.current_view, 'widget_tree'):
-            if hasattr(self.current_view.widget_tree, 'editable'):
-                return self.current_view.widget_tree.editable
-        return False
-
     def new(self, default=True):
-        if (self.current_view
-                and ((self.current_view.view_type == 'tree'
-                        and not (hasattr(self.current_view.widget_tree,
-                                'editable')
-                            and self.current_view.widget_tree.editable))
-                    or self.current_view.view_type == 'graph')):
+        previous_view = self.current_view
+        if self.current_view.view_type == 'calendar':
+            selected_date = self.current_view.get_selected_date()
+        if self.current_view and not self.current_view.editable:
             self.switch_view('form')
             if self.current_view.view_type != 'form':
                 return None
@@ -438,10 +447,11 @@ class Screen(SignalEvent):
             group = self.group
         record = group.new(default)
         group.add(record, self.new_model_position())
+        if previous_view.view_type == 'calendar':
+            previous_view.set_default_date(record, selected_date)
         self.current_record = record
         self.display()
         self.set_cursor(new=True)
-        self.request_set()
         return self.current_record
 
     def new_model_position(self):
@@ -461,8 +471,6 @@ class Screen(SignalEvent):
             self.current_record.cancel()
             if self.current_record.id < 0:
                 self.remove()
-        if self.current_view:
-            self.current_view.cancel()
 
     def save_current(self):
         if not self.current_record:
@@ -483,12 +491,11 @@ class Screen(SignalEvent):
             self.set_cursor()
             self.current_view.display()
             return False
-        self.signal('record-saved')
         if path and obj_id:
             path = path[:-1] + ((path[-1][0], obj_id),)
         self.current_record = self.group.get_by_path(path)
         self.display()
-        self.request_set()
+        self.signal('record-saved')
         return obj_id
 
     def __get_current_view(self):
@@ -535,21 +542,16 @@ class Screen(SignalEvent):
         if written:
             self.group.written(ids)
         if self.parent:
-            self.parent.reload()
+            self.parent.root_parent.reload()
         self.display()
-        self.request_set()
 
     def unremove(self):
-        records = self.current_view.selected_records()
+        records = self.selected_records
         for record in records:
             self.group.unremove(record)
 
     def remove(self, delete=False, remove=False, force_remove=False):
-        records = None
-        if self.current_view.view_type == 'form' and self.current_record:
-            records = [self.current_record]
-        elif self.current_view.view_type == 'tree':
-            records = self.current_view.selected_records()
+        records = self.selected_records
         if not records:
             return
         if delete:
@@ -574,12 +576,13 @@ class Screen(SignalEvent):
 
         if delete:
             for record in records:
-                if record.parent:
-                    record.parent.save(force_reload=False)
                 if record in record.group.record_deleted:
                     record.group.record_deleted.remove(record)
                 if record in record.group.record_removed:
                     record.group.record_removed.remove(record)
+                if record.parent:
+                    # Save parent without deleted children
+                    record.parent.save(force_reload=False)
                 record.destroy()
 
         if idx > 0:
@@ -593,53 +596,77 @@ class Screen(SignalEvent):
             self.current_record = self.group[0]
         self.set_cursor()
         self.display()
-        self.request_set()
         return True
 
+    def copy(self):
+        ids = [r.id for r in self.selected_records]
+        try:
+            new_ids = RPCExecute('model', self.model_name, 'copy', ids, {},
+                context=self.context)
+        except RPCException:
+            return
+        self.load(new_ids)
+
     def set_tree_state(self):
         view = self.current_view
-        if (not CONFIG['client.save_tree_expanded_state']
-                or not self.current_view
-                or self.current_view.view_type != 'tree'
-                or not self.current_view.children_field
+        if (not CONFIG['client.save_tree_state']
+                or not view
+                or view.view_type != 'tree'
                 or not self.group):
             return
+        if id(view) in self.tree_states_done:
+            return
         parent = self.parent.id if self.parent else None
-        expanded_nodes = self.expanded_nodes[parent][view.children_field]
-        if expanded_nodes is None:
+        state = self.tree_states[parent][view.children_field]
+        if state is None:
             json_domain = self.get_tree_domain(parent)
             try:
-                expanded_nodes = RPCExecute('model',
-                    'ir.ui.view_tree_expanded_state', 'get_expanded',
+                expanded_nodes, selected_nodes = RPCExecute('model',
+                    'ir.ui.view_tree_state', 'get',
                     self.model_name, json_domain,
                     self.current_view.children_field)
                 expanded_nodes = json.loads(expanded_nodes)
+                selected_nodes = json.loads(selected_nodes)
             except RPCException:
+                logging.getLogger(__name__).warn(
+                    _('Unable to get view tree state'))
                 expanded_nodes = []
-            self.expanded_nodes[parent][view.children_field] = expanded_nodes
+                selected_nodes = []
+            self.tree_states[parent][view.children_field] = (
+                expanded_nodes, selected_nodes)
+        else:
+            expanded_nodes, selected_nodes = state
         view.expand_nodes(expanded_nodes)
+        view.select_nodes(selected_nodes)
+        self.tree_states_done.add(id(view))
 
-    def save_tree_state(self):
-        view = self.current_view
-        if (not CONFIG['client.save_tree_expanded_state']
-                or not view
-                or view.view_type != 'tree'
-                or not view.children_field
-                or not (self.parent is None
-                    or isinstance(self.parent, Record))):
+    def save_tree_state(self, store=True):
+        if not CONFIG['client.save_tree_state']:
             return
-        parent = self.parent.id if self.parent else None
-        paths = view.get_expanded_paths()
-        self.expanded_nodes[parent][view.children_field] = paths
-        json_domain = self.get_tree_domain(parent)
-        json_paths = json.dumps(paths)
-        try:
-            RPCExecute('model', 'ir.ui.view_tree_expanded_state',
-                'set_expanded', self.model_name, json_domain,
-                self.current_view.children_field, json_paths,
-                process_exception=False)
-        except (TrytonServerError, TrytonServerUnavailable):
-            pass
+        for view in self.views:
+            if view.view_type == 'form':
+                for widgets in view.widgets.itervalues():
+                    for widget in widgets:
+                        if hasattr(widget, 'screen'):
+                            widget.screen.save_tree_state(store)
+            elif (view.view_type == 'tree' and view.children_field):
+                parent = self.parent.id if self.parent else None
+                paths = view.get_expanded_paths()
+                selected_paths = view.get_selected_paths()
+                self.tree_states[parent][view.children_field] = (
+                    paths, selected_paths)
+                if store:
+                    json_domain = self.get_tree_domain(parent)
+                    json_paths = json.dumps(paths)
+                    json_selected_path = json.dumps(selected_paths)
+                    try:
+                        RPCExecute('model', 'ir.ui.view_tree_state', 'set',
+                            self.model_name, json_domain, view.children_field,
+                            json_paths, json_selected_path,
+                            process_exception=False)
+                    except (TrytonServerError, TrytonServerUnavailable):
+                        logging.getLogger(__name__).warn(
+                            _('Unable to set view tree state'))
 
     def get_tree_domain(self, parent):
         if parent:
@@ -650,7 +677,8 @@ class Screen(SignalEvent):
         return json_domain
 
     def load(self, ids, set_cursor=True, modified=False):
-        self.expanded_nodes.clear()
+        self.tree_states.clear()
+        self.tree_states_done.clear()
         self.group.load(ids, modified=modified)
         self.current_view.reset()
         if ids:
@@ -660,7 +688,6 @@ class Screen(SignalEvent):
             self.display()
         if set_cursor:
             self.set_cursor()
-        self.request_set()
 
     def display(self, res_id=None, set_cursor=False):
         if res_id:
@@ -674,10 +701,6 @@ class Screen(SignalEvent):
             else:
                 self.current_record = None
         if self.views:
-            #XXX To remove when calendar will be implemented
-            if self.current_view.view_type == 'calendar' and \
-                    len(self.views) > 1:
-                self.switch_view()
             self.search_active(self.current_view.view_type
                 in ('tree', 'graph', 'calendar'))
             for view in self.views:
@@ -699,7 +722,9 @@ class Screen(SignalEvent):
         if view.view_type == 'tree' and len(self.group):
             start, end = view.widget_tree.get_visible_range()
             vadjustment = view.widget_tree.get_vadjustment()
-            vadjustment.value = vadjustment.value + vadjustment.page_increment
+            vadjustment.value = min(
+                vadjustment.value + vadjustment.page_increment,
+                vadjustment.get_upper())
             store = view.store
             iter_ = store.get_iter(end)
             self.current_record = store.get_value(iter_, 0)
@@ -718,7 +743,7 @@ class Screen(SignalEvent):
                     record = group[idx]
                     break
                 parent = record.parent
-                if not parent:
+                if not parent or record.model_name != parent.model_name:
                     break
                 next = parent.next.get(id(parent.group))
                 while not next:
@@ -731,6 +756,27 @@ class Screen(SignalEvent):
                 record = next
                 break
             self.current_record = record
+        elif view.view_type == 'calendar':
+            record = self.current_record
+            goocalendar = view.children['goocalendar']
+            date = goocalendar.selected_date
+            year = date.year
+            month = date.month
+            start = datetime.datetime(year, month, 1)
+            nb_days = calendar.monthrange(year, month)[1]
+            delta = datetime.timedelta(days=nb_days)
+            end = start + delta
+            events = goocalendar.event_store.get_events(start, end)
+            events.sort()
+            if not record:
+                self.current_record = len(events) and events[0].record
+            else:
+                for idx, event in enumerate(events):
+                    if event.record == record:
+                        next_id = idx + 1
+                        if next_id < len(events):
+                            self.current_record = events[next_id].record
+                        break
         else:
             self.current_record = self.group[0] if len(self.group) else None
         self.set_cursor(reset_view=False)
@@ -743,7 +789,9 @@ class Screen(SignalEvent):
         if view.view_type == 'tree' and len(self.group):
             start, end = view.widget_tree.get_visible_range()
             vadjustment = view.widget_tree.get_vadjustment()
-            vadjustment.value = vadjustment.value - vadjustment.page_increment
+            vadjustment.value = min(
+                vadjustment.value - vadjustment.page_increment,
+                vadjustment.get_lower())
             store = view.store
             iter_ = store.get_iter(start)
             self.current_record = store.get_value(iter_, 0)
@@ -762,16 +810,38 @@ class Screen(SignalEvent):
                         record = children[-1]
             else:
                 parent = record.parent
-                if parent:
+                if parent and record.model_name == parent.model_name:
                     record = parent
             self.current_record = record
+        elif view.view_type == 'calendar':
+            record = self.current_record
+            goocalendar = view.children['goocalendar']
+            date = goocalendar.selected_date
+            year = date.year
+            month = date.month
+            start = datetime.datetime(year, month, 1)
+            nb_days = calendar.monthrange(year, month)[1]
+            delta = datetime.timedelta(days=nb_days)
+            end = start + delta
+            events = goocalendar.event_store.get_events(start, end)
+            events.sort()
+            if not record:
+                self.current_record = len(events) and events[0].record
+            else:
+                for idx, event in enumerate(events):
+                    if event.record == record:
+                        prev_id = idx - 1
+                        if prev_id >= 0:
+                            self.current_record = events[prev_id].record
+                        break
         else:
             self.current_record = self.group[-1] if len(self.group) else None
         self.set_cursor(reset_view=False)
         view.display()
 
-    def sel_ids_get(self):
-        return self.current_view.sel_ids_get()
+    @property
+    def selected_records(self):
+        return self.current_view.selected_records
 
     def id_get(self):
         if not self.current_record:
@@ -789,10 +859,24 @@ class Screen(SignalEvent):
         self.current_record.on_change(fieldname, attr)
         self.display()
 
-    def request_set(self):
-        if self.model_name == 'res.request':
-            from tryton.gui.main import Main
-            Main.get_main().request_set()
+    def get_buttons(self):
+        'Return active buttons for the current view'
+        def is_active(record, button):
+            if record.group.readonly or record.readonly:
+                return False
+            states = record.expr_eval(button.attrs.get('states', {}))
+            return not (states.get('invisible') or states.get('readonly'))
+
+        if not self.selected_records:
+            return []
+
+        buttons = self.current_view.get_buttons()
+
+        for record in self.selected_records:
+            buttons = [b for b in buttons if is_active(record, b)]
+            if not buttons:
+                break
+        return buttons
 
     def button(self, button):
         'Execute button on the current record'
@@ -801,16 +885,76 @@ class Screen(SignalEvent):
         record = self.current_record
         if not record.save(force_reload=False):
             return
-        context = record.context_get()
+        ids = [r.id for r in self.selected_records]
         try:
-            action_id = RPCExecute('model', self.model_name, button['name'],
-                [record.id], context=context)
+            action = RPCExecute('model', self.model_name, button['name'],
+                ids, context=self.context)
         except RPCException:
-            action_id = None
-        if action_id:
-            Action.execute(action_id, {
+            action = None
+        self.reload(ids, written=True)
+        if isinstance(action, basestring):
+            self.client_action(action)
+        elif action:
+            Action.execute(action, {
                     'model': self.model_name,
                     'id': record.id,
-                    'ids': [record.id],
-                    }, context=context)
-        self.reload([record.id], written=True)
+                    'ids': ids,
+                    }, context=self.context)
+
+    def client_action(self, action):
+        access = MODELACCESS[self.model_name]
+        if action == 'new':
+            if access['create']:
+                self.new()
+        elif action == 'delete':
+            if access['delete']:
+                self.remove(delete=not self.parent,
+                    force_remove=not self.parent)
+        elif action == 'remove':
+            if access['write'] and access['read'] and self.parent:
+                self.remove(remove=True)
+        elif action == 'copy':
+            if access['create']:
+                self.copy()
+        elif action == 'next':
+            self.display_next()
+        elif action == 'previous':
+            self.display_prev()
+        elif action == 'close':
+            from tryton.gui import Main
+            Main.get_main().sig_win_close()
+        elif action.startswith('switch'):
+            _, view_type = action.split(None, 1)
+            self.switch_view(view_type=view_type)
+        elif action == 'reload menu':
+            from tryton.gui import Main
+            RPCContextReload(Main.get_main().sig_win_menu)
+        elif action == 'reload context':
+            RPCContextReload()
+
+    def get_url(self):
+        query_string = []
+        if self.domain:
+            query_string.append(('domain', json.dumps(self.domain,
+                        cls=JSONEncoder)))
+        if self.context:
+            query_string.append(('context', json.dumps(self.context,
+                        cls=JSONEncoder)))
+        path = [rpc._DATABASE, 'model', self.model_name]
+        view_ids = [v.view_id for v in self.views] + self.view_ids
+        if self.current_view.view_type != 'form':
+            search_string = self.screen_container.get_text()
+            search_value = self.domain_parser.parse(search_string)
+            if search_value:
+                query_string.append(('search_value', json.dumps(search_value,
+                            cls=JSONEncoder)))
+        elif self.current_record and self.current_record.id > -1:
+            path.append(str(self.current_record.id))
+            i = view_ids.index(self.current_view.view_id)
+            view_ids = view_ids[i:] + view_ids[:i]
+        if view_ids:
+            query_string.append(('views', json.dumps(view_ids)))
+        query_string = urllib.urlencode(query_string)
+        return urlparse.urlunparse(('tryton',
+                '%s:%s' % (rpc._HOST, rpc._PORT),
+                '/'.join(path), query_string, '', ''))
