@@ -5,7 +5,7 @@ import glib
 import gettext
 import os
 import tempfile
-from tryton.common import file_selection, Tooltips, file_open
+from tryton.common import file_selection, Tooltips, file_open, slugify
 from tryton.config import PIXMAPS_DIR
 from interface import WidgetInterface
 import urllib
@@ -48,7 +48,6 @@ class Image(WidgetInterface):
             self.but_add.set_image(img_add)
             self.but_add.set_relief(gtk.RELIEF_NONE)
             self.but_add.connect('clicked', self.sig_add)
-            self.but_add.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_add, _('Select an Image...'))
             hbox.pack_start(self.but_add, expand=False, fill=False)
 
@@ -60,7 +59,6 @@ class Image(WidgetInterface):
                 self.but_open.set_image(img_open)
                 self.but_open.set_relief(gtk.RELIEF_NONE)
                 self.but_open.connect('clicked', self.sig_open)
-                self.but_open.connect('enter-notify-event', self.enter)
                 self.tooltips.set_tip(self.but_open, _('Open...'))
                 hbox.pack_start(self.but_open, expand=False, fill=False)
             else:
@@ -73,7 +71,6 @@ class Image(WidgetInterface):
             self.but_save_as.set_image(img_save_as)
             self.but_save_as.set_relief(gtk.RELIEF_NONE)
             self.but_save_as.connect('clicked', self.sig_save_as)
-            self.but_save_as.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_save_as, _('Save As...'))
             hbox.pack_start(self.but_save_as, expand=False, fill=False)
 
@@ -84,7 +81,6 @@ class Image(WidgetInterface):
             self.but_remove.set_image(img_remove)
             self.but_remove.set_relief(gtk.RELIEF_NONE)
             self.but_remove.connect('clicked', self.sig_remove)
-            self.but_remove.connect('enter-notify-event', self.enter)
             self.tooltips.set_tip(self.but_remove, _('Clear'))
             hbox.pack_start(self.but_remove, expand=False, fill=False)
 
@@ -145,8 +141,11 @@ class Image(WidgetInterface):
         if not self.filename_field:
             return
         dtemp = tempfile.mkdtemp(prefix='tryton_')
-        filename = self.filename_field.get(self.record).replace(
-                os.sep, '_').replace(os.altsep or os.sep, '_')
+        filename = self.filename_field.get(self.record)
+        if not filename:
+            return
+        root, ext = os.path.splitext(filename)
+        filename = ''.join([slugify(root), os.extsep, slugify(ext)])
         file_path = os.path.join(dtemp, filename)
         with open(file_path, 'wb') as fp:
             fp.write(self.field.get_data(self.record))
