@@ -11,6 +11,7 @@ class ViewGraph(ParserView):
             state_widgets, notebooks, cursor_widget, children_field)
         self.view_type = 'graph'
         self.widgets = children
+        self.editable = False
 
     def __getitem__(self, name):
         return None
@@ -20,19 +21,10 @@ class ViewGraph(ParserView):
         for widget in self.widgets.keys():
             self.widgets[widget].destroy()
             del self.widgets[widget]
-        self.widget = None
-        self.screen = None
-        self.state_widgets = None
-
-    def cancel(self):
-        pass
 
     def set_value(self):
         pass
 
-    def sel_ids_get(self):
-        return []
-
     def reset(self):
         pass
 
