@@ -3,7 +3,7 @@
 #This code is inspired by the pycha project
 #(http://www.lorenzogil.com/projects/pycha/)
 from graph import Graph
-from tryton.common import hex2rgb, float_time_to_text, safe_eval
+from tryton.common import hex2rgb, float_time_to_text
 import locale
 import math
 import cairo
@@ -34,7 +34,7 @@ class Line(Graph):
                 if self.xrange == 0:
                     x = 1.0
 
-                if (not bool(safe_eval(yfield2attrs[yfield].get('empty', '1')))
+                if (not bool(int(yfield2attrs[yfield].get('empty', 1)))
                         and yval == 0):
                     continue
 
@@ -50,7 +50,7 @@ class Line(Graph):
         key2interpolation = {}
         for yfield in self.yfields:
             key = yfield.get('key', yfield['name'])
-            key2fill[key] = bool(safe_eval(yfield.get('fill', '0')))
+            key2fill[key] = bool(int(yfield.get('fill', 0)))
             key2interpolation[key] = yfield.get('interpolation', 'linear')
 
         def preparePath(key):
@@ -127,8 +127,8 @@ class Line(Graph):
             cr.move_to(point.x * self.area.w + self.area.x,
                     point.y * self.area.h + self.area.y)
             cr.arc(point.x * self.area.w + self.area.x,
-                    point.y * self.area.h + self.area.y,
-                    3, 0, 2 * math.pi)
+                point.y * self.area.h + self.area.y,
+                3, 0, 2 * math.pi)
             cr.fill()
 
         cr.restore()
@@ -143,17 +143,20 @@ class Line(Graph):
                 cr.move_to(point.x * self.area.w + self.area.x,
                         point.y * self.area.h + self.area.y)
                 cr.arc(point.x * self.area.w + self.area.x,
-                        point.y * self.area.h + self.area.y,
-                        3, 0, 2 * math.pi)
+                    point.y * self.area.h + self.area.y,
+                    3, 0, 2 * math.pi)
                 cr.stroke()
                 cr.set_source_rgb(*self.colorScheme['__highlight'])
                 cr.arc(point.x * self.area.w + self.area.x,
-                        point.y * self.area.h + self.area.y,
-                        3, 0, 2 * math.pi)
+                    point.y * self.area.h + self.area.y,
+                    3, 0, 2 * math.pi)
                 cr.fill()
         cr.restore()
 
     def motion(self, widget, event):
+        if not getattr(self, 'area', None):
+            return
+
         nearest = None
         for point in self.points:
             x = point.x * self.area.w + self.area.x
@@ -244,7 +247,7 @@ class Line(Graph):
     def YLabels(self):
         ylabels = super(Line, self).YLabels()
         if len([x.get('key', x['name']) for x in self.yfields
-            if x.get('widget')]) == len(self.yfields):
+                    if x.get('widget')]) == len(self.yfields):
             conv = None
             float_time = reduce(lambda x, y: x == y and x or False,
                     [x.get('float_time') for x in self.yfields])
