@@ -66,7 +66,7 @@ def sort_model(column, treeview, screen):
             column.arrow.hide()
     store = treeview.get_model()
     unsaved_records = [x for x in store.group if x.id < 0]
-    search_string = screen.screen_container.get_text() or None
+    search_string = screen.screen_container.get_text() or u''
     if screen.search_count == len(store) or unsaved_records or screen.parent:
         ids = screen.search_filter(search_string=search_string, only_ids=True)
         store.sort(ids)
