@@ -2,9 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
-import operator
 import gobject
-import collections
 
 import tryton.common as common
 from tryton.common.domain_parser import quote
@@ -63,6 +61,8 @@ class Selection(gtk.ScrolledWindow):
     def get_value(self):
         values = []
         model, paths = self.treeview.get_selection().get_selected_rows()
+        if not paths:
+            return
         for path in paths:
             iter_ = model.get_iter(path)
             values.append(model.get_value(iter_, 0))
@@ -102,11 +102,9 @@ class ScreenContainer(object):
         self.completion = gtk.EntryCompletion()
         self.completion.set_model(gtk.ListStore(str))
         self.completion.set_text_column(0)
-        self.completion.props.inline_completion = True
-        if hasattr(self.completion.props, 'inline_selection'):
-            self.completion.props.inline_selection = True
-        if hasattr(self.completion.props, 'popup_set_width'):
-            self.completion.props.popup_set_width = False
+        self.completion.props.inline_selection = True
+        self.completion.props.popup_set_width = False
+        self.completion.set_match_func(lambda *a: True)
         self.completion.connect('match-selected', self.match_selected)
         self.search_entry.connect('activate', self.activate)
         self.search_entry.set_completion(self.completion)
@@ -133,8 +131,7 @@ class ScreenContainer(object):
                     False
                 )
 
-            for id_, name, domain in common.VIEW_SEARCH[
-                    self.screen.model_name]:
+            for id_, name, domain in self.bookmarks():
                 menuitem = gtk.MenuItem(name)
                 menuitem.connect('activate', self.bookmark_activate, domain)
                 menu.add(menuitem)
@@ -227,8 +224,7 @@ class ScreenContainer(object):
 
     def set_screen(self, screen):
         self.screen = screen
-        self.but_bookmark.set_sensitive(
-            bool(common.VIEW_SEARCH[screen.model_name]))
+        self.but_bookmark.set_sensitive(bool(list(self.bookmarks())))
         self.bookmark_match()
 
     def show_filter(self):
@@ -287,6 +283,11 @@ class ScreenContainer(object):
         self.search_entry.set_text(value)
         self.bookmark_match()
 
+    def bookmarks(self):
+        for id_, name, domain in common.VIEW_SEARCH[self.screen.model_name]:
+            if self.screen.domain_parser.stringable(domain):
+                yield id_, name, domain
+
     def bookmark_activate(self, menuitem, domain):
         self.set_text(self.screen.domain_parser.string(domain))
         self.do_search()
@@ -299,7 +300,7 @@ class ScreenContainer(object):
         self.search_entry.set_icon_sensitive(gtk.ENTRY_ICON_SECONDARY,
             bool(current_text))
         icon_stock = self.search_entry.get_icon_stock(gtk.ENTRY_ICON_SECONDARY)
-        for id_, name, domain in common.VIEW_SEARCH[self.screen.model_name]:
+        for id_, name, domain in self.bookmarks():
             text = self.screen.domain_parser.string(domain)
             domain = self.screen.domain_parser.parse(text.decode('utf-8'))
             if (text == current_text
@@ -347,6 +348,8 @@ class ScreenContainer(object):
 
     def match_selected(self, completion, model, iter):
         def callback():
+            if not self.search_entry.props.window:
+                return
             self.update()
             self.search_entry.emit('changed')
         gobject.idle_add(callback)
@@ -363,8 +366,12 @@ class ScreenContainer(object):
             self.search_entry.grab_focus()
 
     def key_press(self, widget, event):
-        gobject.idle_add(self.update)
-        gobject.idle_add(self.bookmark_match)
+        def keypress():
+            if not self.search_entry.props.window:
+                return
+            self.update()
+            self.bookmark_match()
+        gobject.idle_add(keypress)
 
     def icon_press(self, widget, icon_pos, event):
         if icon_pos == 1:
@@ -385,8 +392,7 @@ class ScreenContainer(object):
                 common.VIEW_SEARCH.remove(model_name, id_)
             # Refresh icon and bookmark button
             self.bookmark_match()
-            self.but_bookmark.set_sensitive(
-                bool(common.VIEW_SEARCH[model_name]))
+            self.but_bookmark.set_sensitive(bool(list(self.bookmarks())))
 
     def focus_in(self, widget, event):
         self.update()
@@ -404,13 +410,13 @@ class ScreenContainer(object):
             text = ''
             for label, entry in self.search_table.fields:
                 if isinstance(entry, gtk.ComboBox):
-                    value = quote(entry.get_active_text())
+                    value = quote(entry.get_active_text()) or None
                 elif isinstance(entry, (Dates, Selection)):
                     value = entry.get_value()
                 else:
-                    value = quote(entry.get_text())
-                if value:
-                    text += label + ' ' + value + ' '
+                    value = quote(entry.get_text()) or None
+                if value is not None:
+                    text += quote(label) + ': ' + value + ' '
             self.set_text(text)
             self.last_search_text = self.get_text()
             self.do_search()
@@ -428,16 +434,11 @@ class ScreenContainer(object):
             self.search_window.set_title('Tryton')
             self.search_window.set_icon(TRYTON_ICON)
             self.search_window.set_decorated(False)
-            if hasattr(self.search_window, 'set_deletable'):
-                self.search_window.set_deletable(False)
+            self.search_window.set_deletable(False)
             self.search_window.connect('key-press-event', key_press)
             vbox = gtk.VBox()
             fields = [f for f in self.screen.domain_parser.fields.itervalues()
                 if f.get('searchable', True)]
-            if (not hasattr(collections, 'OrderedDict')
-                    or not isinstance(self.screen.domain_parser.fields,
-                        collections.OrderedDict)):
-                fields.sort(key=operator.itemgetter('string'))
             self.search_table = gtk.Table(rows=len(fields), columns=2)
             self.search_table.set_homogeneous(False)
             self.search_table.set_border_width(5)
@@ -479,11 +480,12 @@ class ScreenContainer(object):
                     entry.connect('activate', lambda *a: search())
                 self.search_table.attach(entry, 1, 2, i, i + 1,
                     yoptions=yoptions)
-                self.search_table.fields.append((field['string'] + ':', entry))
+                self.search_table.fields.append((field['string'], entry))
 
             scrolled = gtk.ScrolledWindow()
             scrolled.add_with_viewport(self.search_table)
             scrolled.set_shadow_type(gtk.SHADOW_NONE)
+            scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
             vbox.pack_start(scrolled, expand=True, fill=True)
             find_button = gtk.Button(_('Find'))
             find_button.connect('clicked', lambda *a: search())
@@ -498,26 +500,19 @@ class ScreenContainer(object):
             self.search_window.add(vbox)
             vbox.show_all()
 
-            # Disable scrolling:
-            scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_NEVER)
-            # See what changed:
-            new_size = self.search_window.size_request()
-            # Reenable scrolling:
-            scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+            new_size = map(sum, zip(self.search_table.size_request(),
+                    scrolled.size_request()))
             self.search_window.set_default_size(*new_size)
 
         parent = button.get_toplevel()
-        button_x, button_y = button.translate_coordinates(parent,
-            *parent.window.get_origin())
+        button_x, button_y = button.translate_coordinates(parent, 0, 0)
         button_allocation = button.get_allocation()
 
-        # Resize the window to not be out of the screen
+        # Resize the window to not be out of the parent
         width, height = self.search_window.get_default_size()
-        screen = self.search_window.get_screen()
-        screen_width = screen.get_width()
-        screen_height = screen.get_height()
-        delta_width = screen_width - (button_x + width)
-        delta_height = screen_height - (button_y + button_allocation.height
+        allocation = parent.get_allocation()
+        delta_width = allocation.width - (button_x + width)
+        delta_height = allocation.height - (button_y + button_allocation.height
             + height)
         if delta_width < 0:
             width += delta_width
@@ -526,8 +521,8 @@ class ScreenContainer(object):
         self.search_window.resize(width, height)
 
         # Move the window under the button
-        self.search_window.move(button_x,
-            button_y + button_allocation.height)
+        x, y = button.window.get_origin()
+        self.search_window.move(x, y + button_allocation.height)
 
         from tryton.gui.main import Main
         page = Main.get_main().get_page()
